% stats is a matrix where each row corresponds to a different pokemon. Each
% column is a different base stat (HP/atk/def/spatk/spdef/speed)

% names is a cell array where entry i is the name of the pokemon for row i
% in stats

% Remove nans
names = names(~isnan(stats(:,1)));
stats(isnan(stats(:,1)),:) = [];

max_num = size(stats,1);
diff = zeros(max_num,max_num);
for i = 1:1:max_num
    for j = (min(i+1,max_num)):1:max_num
        for k = 1:1:6
            diff(i,j) = diff(i,j) + (stats(i,k) - stats(j,k))^2;
        end
        % Make values distinct. Gives priority
        if diff(i,j) ~= 0
            diff(i,j) = diff(i,j) + i*10^-3 + j * 10^-6;
        end
    end
end

% Filter off zeros:
for i = 1:1:max_num
    for j = 1:1:max_num
        if diff(i,j) == 0
            diff(i,j) = 10^5;
        end
    end
end

% Now display best matches

for i = 1:1:300
    current_min = min(diff(:));
    [r,c] = find(diff==min(diff(:)));
    % Remove the extra decimal added to distinguish
    current_min = floor(current_min);
    fprintf('%g Point difference: %s + %s\n',current_min(1),names{r},names{c})
    
    % Blank it:
    diff(r,c) = 10^5;
end

% Separately, perform a k-mean cluster analysis.
%
num_clusters = 100; name_per_line = 5;
%

% REMOVE from final release
% For clusters, compare relative base stat distributions (normalize by BST)
%stats2 = stats(:,1:6) ./ stats(:,7) * mean(stats(:,7));

[kmean_indices, centroids] = kmeans(stats(:,1:6), num_clusters);
% Optimize k-mean cluster
kmean_best = kmean_indices; centr_best = centroids; rms_error_best = 10^20;
for i = 1:1:100
    rms_error = 0;
    for j = 1:1:max_num
        rms_error = rms_error + ...
            sum((stats(i,1:6) - centroids(kmean_indices(j),1:6)).^2);
    end
    if rms_error < rms_error_best
        kmean_best = kmean_indices; centr_best = centroids;
        rms_error_best = rms_error;
    end
end
kmean_indices = kmean_best; centroids = centr_best;

fprintf('\nK-mean cluster analysis:')
% Round down centroids for nicer display
centroids = round(centroids, 2);
unclustered = {};
for i = 1:1:num_clusters
    cluster_names = names(kmean_indices == i);
    if size(cluster_names,1) == 1
        unclustered(size(unclustered,1) + 1,1) = cluster_names(1,1);
        continue
    end
    
    fprintf('\nCluster %i: Stats: %g/%g/%g/%g/%g/%g.',i,...
        centroids(i,1),centroids(i,2),centroids(i,3),...
        centroids(i,4),centroids(i,5),centroids(i,6));
    
    % Determine RMS distance to centroid:
    rms_centroid = 0;
    cluster_stats = stats(kmean_indices == i,1:6);
    for j = 1:1:size(cluster_stats,1)
        rms_centroid = rms_centroid + sum((centroids(i,:) - cluster_stats(j,:)).^2);
    end
    rms_centroid = rms_centroid / size(cluster_stats,1);
    fprintf('\n     %i Members. Mean Square Diff = %g\n',...
        size(cluster_names,1),rms_centroid);    
    
    % Add blank spaces to end of cluster array
    for j = 1:1:name_per_line
        if mod(size(cluster_names,1),name_per_line) ~= 0
            cluster_names{size(cluster_names,1) + 1,1} = '';
        end
    end
    % Format names of members into a nice string.
    name_str = '';
    for j = 1:name_per_line:size(cluster_names,1)
        disp(sprintf(['%s,' repmat(' %s,',1,name_per_line - 1)],...
            cluster_names{j:(j + name_per_line - 1),1}));
    end
    disp(name_str)
end

fprintf('\nUnclustered Pokemon:\n')
% Add blank spaces to end of cluster array
for j = 1:1:name_per_line
    if mod(size(unclustered,1),name_per_line) ~= 0
        unclustered{size(unclustered,1) + 1,1} = '';
    end
end
for j = 1:name_per_line:size(unclustered,1)
    fprintf(['%s,' repmat(' %s,',1,name_per_line - 1) '\n'],...
        unclustered{j:(j + name_per_line - 1)})
end